/*
*===========================================================
*        File Name: adc.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ADC.h"
#include "hardware.h"


static void adc_init_bat_volt_curr(unsigned char updrate_index);
static void adc_init_bat_temp_vdac(unsigned char updrate_index);
static void adc_init_bat_temp_sync(unsigned char updrate_index);
static void adc_iext_curr_on(void);
static void adc_iext_curr_off(void);
static void adc_sync_start(void);
static void adc_odr_tsettle(unsigned short ADCFLT, float* fADC, float* tsettle);
static unsigned int adc_raw_ch0(void);
static unsigned int adc_raw_ch1(void);

ADC_T adc ={
	.ADC_UPD_RATE = { 3.53 ,30.0  ,50.0  ,100.0 ,488.0 ,976.0 ,1953.0,3906.0},		//recommended ADC Update Rate with verified performance.
	.ADCFLT_VALUE = {0x8D7C,0x007E,0x007D,0x004D,0x100F,0x1007,0x1003,0x1001},		//recommended ADCFTL Value	
	.init_bat_volt_curr = adc_init_bat_volt_curr,
	.init_bat_temp_vdac = adc_init_bat_temp_vdac,
	.init_bat_temp_sync = adc_init_bat_temp_sync,
	.sync_start = adc_sync_start,
	.iext_curr_on = adc_iext_curr_on,
	.iext_curr_off = adc_iext_curr_off,
	.odr_tsettle = adc_odr_tsettle,
	.raw_ch0 = adc_raw_ch0,
	.raw_ch1 = adc_raw_ch1
};

static void adc_init_bat_volt_curr(unsigned char updrate_index){
//ADC0 Configuration	
	pADI_ADC0->ADCCFG = 								//ADC0CFG and ADC1CFG are the same physical register,
	(0x0 << 15) + //SIMU = 0;
	(0x0 << 13) + //BOOST30 = 0;
	(0x0 << 8)  + //VBIAS PINSEL = 0;	Disable
	(0x0 << 7)  + //GNDSWON = 0;	OPen
	(0x0 << 6)  + //GNDSWRESEN = 0; Disable 20kO resistor
	(0x2 << 0);	  //EXTBUF: The VREF+ and EXTREF2IN+ inputs are selected as buffer inputs
	pADI_ADC0->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Bypolar
	(0xF<<14) + //BUFPOWN = 1; BUFPOWP = 1; BUFBYPP = 1; BUFBYPN = 1
	(0x0<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Current source off. 
	PIN_VBP +
	PIN_VBN ;
	pADI_ADC0->MSKI = 0x0;// Disable Valid conversion Interrupt	
	pADI_ADC0->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC0->MDE = 
	(0x0<<4) + //PGA Gain = 1;
	//(0x1<<4) + //PGA Gain = 2;
	(0x0<<3) + //Modulator gain of 2 Disabled
	//(0x3);     //ADCMD: Idle Mode
	(0x2);     //ADCMD: Single Conversion Mode
	
//ADC1 Configuration
	//ADC0CFG and ADC1CFG are the same physical register
	pADI_ADC1->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Bypolar
	(0x0<<14) + //BUFPOWN = 0; BUFPOWP = 0; BUFBYPP = 0; BUFBYPN = 0
	(0x0<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Current source off. 
	PIN_IBP +
	PIN_IBN ;
	pADI_ADC1->MSKI = 0x0;// Disable Valid conversion Interrupt
	pADI_ADC1->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC1->MDE = 
	(0x1<<4) + //PGA Gain = 2;
	(0x0<<3) + //Modulator gain of 2 Disabled
	(0x2);     //ADCMD: Single Conversion Mode
}

static void adc_init_bat_temp_vdac(unsigned char updrate_index){
//ADC0 Configuration	
	pADI_ADC0->ADCCFG = 								//ADC0CFG and ADC1CFG are the same physical register,
	(0x0 << 15) + //SIMU = 0;
	(0x0 << 13) + //BOOST30 = 0;
	(0x0 << 8) + //VBIAS PINSEL = 0;	Disable
	(0x0 << 7) + //GNDSWON = 0;	OPen
	(0x0 << 6) + //GNDSWRESEN = 0; Disable 20kO resistor
	(0x2 << 0);	//EXTBUF: The VREF+ and EXTREF2IN+ inputs are selected as buffer inputs
	pADI_ADC0->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Byipolar
	(0xF<<14) + //BUFPOWN = 1; BUFPOWP = 1; BUFBYPP = 1; BUFBYPN = 1
	(0x0<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Current source off. 
	PIN_VDACFB +
	ADCCON_ADCCN_AGND;
	pADI_ADC0->MSKI = 0x0;// Disable Valid conversion Interrupt
	pADI_ADC0->FLT = pADI_ADC0->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC0->MDE = 
	(0x0<<4) + //PGA Gain = 1;		
	(0x0<<3) + //Modulator gain of 2 Disabled
	(0x2);     //ADCMD: Single Conversion Mode
//ADC1 Configuration
	//ADC0CFG and ADC1CFG are the same physical register,
	pADI_ADC1->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Bypolar
	(0x0<<14) + //BUFPOWN = 0; BUFPOWP = 0; BUFBYPP = 0; BUFBYPN = 0
	(0x2<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Current source off. 
	PIN_THRP +
	PIN_THRN ;
	pADI_ADC1->MSKI = 0x0;// Enable Valid conversion Interrupt
	pADI_ADC1->FLT = pADI_ADC0->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC1->MDE = 
	(0x1<<4) + //PGA Gain = 2; 2.2K reference compared to 10K@25 degC Thermister. Gain should be changed depend on the current temeparture.
	(0x0<<3) + //Modulator gain of 2 Disabled
	(0x2);     //ADCMD: Single Conversion Mode	
}

static void adc_init_bat_temp_sync(unsigned char updrate_index){
//ADC0 Configuration	
	pADI_ADC0->ADCCFG = 								//ADC0CFG and ADC1CFG are the same physical register,
	(0x0 << 15) + //SIMU = 0;
	(0x0 << 13) + //BOOST30 = 0;
	(0x0 << 8) + //VBIAS PINSEL = 0;	Disable
	(0x0 << 7) + //GNDSWON = 0;	OPen
	(0x0 << 6) + //GNDSWRESEN = 0; Disable SW
	(0x0 << 0);	 //EXTBUF: The VREF+ and EXTREF2IN+ inputs are selected as buffer inputs
	pADI_ADC0->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Byipolar
	(0xF<<14) + //BUFPOWN = 0; BUFPOWP = 0 BUFBYPP = 0; BUFBYPN = 0
	(0x3<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Diagnostic Current source off. 
	PIN_THRREF +
	//ADCCON_ADCCN_AGND;
	PIN_THRREFGND;
	pADI_ADC0->MSKI = 0x0;// Disable Valid conversion Interrupt
	pADI_ADC0->FLT = pADI_ADC0->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC0->MDE = 
	(0x0<<4) + //PGA Gain = 1;		
	(0x0<<3) + //Modulator gain of 2 Disabled
	(0x2);     //ADCMD: Single Conversion Mode
//ADC1 Configuration
	//ADC0CFG and ADC1CFG are the same physical register,
	pADI_ADC1->CON = 
	(0x0<<19) + //ADCEN = 0;
	(0x0<<18) + //ADCCODE = 0; Bypolar
	(0x0<<14) + //BUFPOWN = 0; BUFPOWP = 0; BUFBYPP = 0; BUFBYPN = 0
	(0x3<<12) + //ADCREF = 00; (00: INTREF-AGND. ADCREF = 01: EXTREF. ADCREF = 10: EXTREF2IN. ADCREF = 11: AVDD-AGND) 
	(0x0<<10) + //ADCDIAG = 00;  Current source off. 
//	PIN_THRREF +
//	ADCCON_ADCCN_AGND;
	PIN_THRP +
	PIN_THRN ;
	pADI_ADC1->MSKI = 0x0;// Enable Valid conversion Interrupt
	pADI_ADC1->FLT = pADI_ADC0->FLT = ((updrate_index <= 7)? adc.ADCFLT_VALUE[updrate_index]:adc.ADCFLT_VALUE[7]);
	pADI_ADC1->MDE = 
	(0x2<<4) + //PGA Gain = 2; 2.2K reference compared to 10K@25 degC Thermister. Gain should be changed depend on the current temeparture.
	(0x0<<3) + //Modulator gain of 2 Disabled
	(0x2);     //ADCMD: Single Conversion Mode
};
static void adc_iext_curr_on(void){
// 	pADI_ANA->IEXCDAT = 0x8;//2K Protection Resistor, 2.2K Reference Resistor, Thermister 10K: 1~50k(-10degC ~ 85 degC), Total Resistance = 60k ==> Iext <= 3.3V/60k = 55uA.
if(1){//10uA
	IexcCfg(IEXCCON_PD_off, IEXCCON_REFSEL_Int, IEXCCON_IPSEL1_AIN6, IEXCCON_IPSEL0_Off);
	IexcDat(IEXCDAT_IDAT_0uA, IDAT0En);		
}
if(0){//20uA	
	IexcCfg(IEXCCON_PD_off, IEXCCON_REFSEL_Int, IEXCCON_IPSEL1_AIN6, IEXCCON_IPSEL1_AIN6);
	IexcDat(IEXCDAT_IDAT_0uA, IDAT0En);	
}
if(0){//50uA
	IexcCfg(IEXCCON_PD_off, IEXCCON_REFSEL_Int, IEXCCON_IPSEL1_AIN6, IEXCCON_IPSEL0_Off);
	IexcDat(IEXCDAT_IDAT_50uA, IDAT0Dis);	
}
if(0){//100uA
	IexcCfg(IEXCCON_PD_off, IEXCCON_REFSEL_Int, IEXCCON_IPSEL1_AIN6, IEXCCON_IPSEL0_AIN6);
	IexcDat(IEXCDAT_IDAT_50uA, IDAT0Dis);	
}
}
static void adc_iext_curr_off(void){
	IexcCfg(IEXCCON_PD_En, IEXCCON_REFSEL_Int, IEXCCON_IPSEL1_Off, IEXCCON_IPSEL0_Off);
	IexcDat(IEXCDAT_IDAT_0uA, IDAT0Dis);	
};
static unsigned int adc_raw_ch0(void){
	if(pADI_ADC0->STA&ADCMSKI_RDY){
		pADI_ADC0->STA &=(~ADCMSKI_RDY);
		return(pADI_ADC0->DAT);
	}
	else
		return(0);
}
static unsigned int adc_raw_ch1(void){
	if(pADI_ADC1->STA&ADCMSKI_RDY){
		pADI_ADC1->STA &=(~ADCMSKI_RDY);
		return(pADI_ADC1->DAT);
	}
	else
		return(0);
}

static void adc_sync_start(void){
	pADI_ADC1->ADCCFG |= 0x8000;
}
 
static void adc_odr_tsettle(unsigned short ADCFLT, float* fADC, float* tsettle){
//	unsigned short ADCFLT = 0;
	ADCFLT_T adcflt;
	unsigned short SF = 0;
	
	adcflt.Val = ADCFLT;
	SF = adcflt.Bits.SF;	
	switch(adcflt.Bits.SF){		//SF Reassignment
		case 124:
		{	
			SF = 130;
			break;
		}
		case 125:
		{	
			SF = 158;
			break;
		}
		case 126:
		{	
			SF = 260;
			break;
		}
		case 127:
		{	
			SF = 316;
			break;
		}
		default:
			break;
	}	
	if(adcflt.Bits.SINC4EN){
		if(adcflt.Bits.CHOP){
			*fADC = (125000.0/((16*(SF+1)*4)+3));
			*tsettle = (2.0/(*fADC));
		}
		else{
			*fADC = (125000.0/(16*(SF+1)));
			if(adcflt.Bits.RAVG2){
				*tsettle = (5.0/(*fADC));
			}
			else{
				*tsettle = (4.0/(*fADC));
			}
		}
	}
	else{
		if(adcflt.Bits.CHOP){
			*fADC = (125000.0/(16*(SF+1)*(3+adcflt.Bits.AF)+3));
			*tsettle = (3.0/(*fADC));
		}
		else if(adcflt.Bits.AF){
			*fADC = (125000.0/(16*(SF+1)*(3+adcflt.Bits.AF)));
			if(adcflt.Bits.RAVG2){
				*tsettle = (2.0/(*fADC));				
			}
			else{
				*tsettle = (1.0/(*fADC));				
			}
		}
		else{
			*fADC = (125000.0/(16*(SF+1)));
			if(adcflt.Bits.RAVG2){
				*tsettle = (4.0/(*fADC));				
			}
			else{
				*tsettle = (3.0/(*fADC));				
			}			
		}
	}	
}

