
/*
*===========================================================
*        File Name: dac.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hardware.h"
#include "dac.h"

#define VDAC_FACTOR_12BIT (4095/1.2)
#define VDAC_FACTOR_16BIT (65535/1.2)

static void dac_init(void);
static void dac12_wr_raw(uint16_t raw);
static void dac16_wr_raw(uint16_t raw);
static void dac12_wr_volt(float vdac);
static void dac16_wr_volt(float vdac);

DAC_T dac = {
	.init = dac_init,
	.wr_raw12 = dac12_wr_raw,
	.wr_raw16 = dac16_wr_raw,
	.wr_volt12 = dac12_wr_volt,
	.wr_volt16 = dac16_wr_volt
};

static void dac_init(void){
	pADI_DAC->DACCON = 0x00; //Get DAC output Power Down Mode and made Condiguration
  DacCfg(DACCON_CLR_Off,DACCON_RNG_IntVref,DACCON_CLK_HCLK,DACCON_MDE_12bit);           //DAC Output Enable, DAC range 0 to 1.2V, Buffer On, Normode Voltage Mode 12-bit mode
//   DacDma(0,DACCON_DMAEN_On);                        // Enable DAC DMA
//	DacDma(0,DACCON_DMAEN_Off);                        // Disable DAC DMA	
};
static void dac12_wr_raw(uint16_t raw){
	DACDAT_T dactemp;
	dactemp.Val = 0;
	dactemp.BITs.DAT12 = (raw&0x0FFF);
	pADI_DAC->DACDAT = dactemp.Val;
};
static void dac16_wr_raw(uint16_t raw){
	DACDAT_T dactemp;
	dactemp.Val = 0;
	dactemp.BITs.DAT12 = (uint32_t)(raw&0x0FFF);
	dactemp.BITs.DAT16 = (uint32_t)((raw&0xF000)>>12);
	pADI_DAC->DACDAT = dactemp.Val;	
};
static void dac12_wr_volt(float vdac){
	uint16_t rawtemp = 0;
	vdac = (vdac > 1.2)? 1.2:vdac;
	rawtemp = (uint16_t) vdac*VDAC_FACTOR_12BIT;
	dac12_wr_raw(rawtemp);
};
static void dac16_wr_volt(float vdac){
	uint16_t rawtemp = 0;
	vdac = (vdac > 1.2)? 1.2:vdac;
	rawtemp = (uint16_t) vdac*VDAC_FACTOR_16BIT;
	dac16_wr_raw(rawtemp);	
};
