
/*
*===========================================================
*        File Name: uart.c
*          Version: A1
*          Created: 2015-8-14
*         Revision: 00
*         Compiler: ARMCC.exe V5.03.0.76
*           Author: 
*          Company: Analog Devices Inc
*===========================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uart.h"
#include "hardware.h"
#include "ctrlport.h"
#include "timer1.h"
//#include "cmd.h"

static float uart_cal_chartime(unsigned int baudrate);
static float uart_cal_framelength(void);

static void uart_send_dbuf_wr(unsigned short data);
static unsigned char uart_send_dbuf_rd(void);
static void uart_send_lbuf_wr(unsigned short Data);
static unsigned char uart_send_lbuf_rd(void);
static void uart_receive_dbuf_wr( unsigned short Data);
static unsigned char uart_receive_dbuf_rd(void);
static void uart_receive_lbuf_wr( unsigned short Data);
static unsigned char uart_receive_lbuf_rd(void);
static void uart_dmasend(void);

static void uart_send_string_int(void);
static void uart_modbusend_string_int(void);
static void uart_normalsend_string_int(void);
static unsigned char uart_init(void);
static unsigned char uart_dmainit(void);
static unsigned char uart_setbaudrate(unsigned int baudrate);
extern void UART_Int_Handler(void);
extern void DMA_UART_TX_Int_Handler(void);
extern void DMA_UART_RX_Int_Handler(void);
static int UrtCfgX(ADI_UART_TypeDef *pPort, int iBaud, int iBits, int iFormat);
static unsigned short uart_send_crc(SENDDDD_DBUF_PTR_T phead, unsigned short length);
static unsigned short uart_receive_crc(RECEIVE_DBUF_PTR_T phead, unsigned short length);

unsigned char szTemp[64] = "abcdefg";                       // Used to store string before printing to UART
unsigned char szPacketIn[32];	                       // Used to store received bytes from UART

UART_T uart = {
	.BAUDRATE_VALID = {9600, 19200, 38400, 57600, 115200},
	.send_dbuf = {
		.Status.BITs.Empty = 1
	},
	.send_lbuf = {
		.Status.BITs.Empty = 1
	},	
	.send_dbuf_wr = uart_send_dbuf_wr,
	.send_dbuf_rd = uart_send_dbuf_rd,
	.send_lbuf_wr = uart_send_lbuf_wr,
	.send_lbuf_rd = uart_send_lbuf_rd,	
	
	.receive_dbuf = {
		.Status.BITs.Empty = 1
	},
	.receive_lbuf = {
		.Status.BITs.Empty = 1
	},		
	.send_length = 0,
	
	.cal_chartime = uart_cal_chartime,
	.cal_framelength = uart_cal_framelength,
	
	.receive_dbuf_wr = uart_receive_dbuf_wr,
	.receive_dbuf_rd = uart_receive_dbuf_rd,
	.receive_lbuf_wr = uart_receive_lbuf_wr,
	.receive_lbuf_rd = uart_receive_lbuf_rd,
	
	.send_crc = uart_send_crc,
	.receive_crc = uart_receive_crc,
	
	.send_string_int = uart_send_string_int,
	.modbusend_string_int = uart_modbusend_string_int,
	.normalsend_string_int = uart_normalsend_string_int,
	.init = uart_init,
	.dmainit = uart_dmainit,
	.dmasend = uart_dmasend,
	.setbaudrate = uart_setbaudrate,
	.cfg = UrtCfgX,	
	.Int_Handler = UART_Int_Handler,
	.status = UART_IDLE
};
//Buffer Operation
static void uart_send_dbuf_wr(unsigned short Data){
	if(uart.send_dbuf.Status.BITs.Full == 0){
		uart.send_dbuf.Data[uart.send_dbuf.pWrite.Val] = Data;
		uart.send_dbuf.Status.BITs.Empty = 0;
		uart.send_dbuf.pWrite.Val++;
		if(uart.send_dbuf.pRead.Val == uart.send_dbuf.pWrite.Val)
			uart.send_dbuf.Status.BITs.Full = 1;
		uart.send_dbuf.Wused.Val++;
	}
}
static unsigned char uart_send_dbuf_rd(void){
	unsigned char temp = 0;
	if(uart.send_dbuf.Status.BITs.Empty == 0){
		temp = uart.send_dbuf.Data[uart.send_dbuf.pRead.Val];
		uart.send_dbuf.Status.BITs.Full = 0;	
		uart.send_dbuf.pRead.Val++;
		if(uart.send_dbuf.pRead.Val == uart.send_dbuf.pWrite.Val){
			uart.send_dbuf.Status.BITs.Empty = 1;
		}
		uart.send_dbuf.Wused.Val--;
	}
	return(temp);
}
static void uart_send_lbuf_wr(unsigned short Data){
	if(uart.send_lbuf.Status.BITs.Full == 0){
		uart.send_lbuf.Data[uart.send_lbuf.pWrite.Val] = Data;
		uart.send_lbuf.Status.BITs.Empty = 0;
		uart.send_lbuf.pWrite.Val++;
		if(uart.send_lbuf.pRead.Val == uart.send_lbuf.pWrite.Val)
			uart.send_lbuf.Status.BITs.Full = 1;
		uart.send_lbuf.Wused.Val++;
	}
}
static unsigned char uart_send_lbuf_rd(void){
	unsigned char temp = 0;
	if(uart.send_lbuf.Status.BITs.Empty == 0){
		temp = uart.send_lbuf.Data[uart.send_lbuf.pRead.Val];
		uart.send_lbuf.Status.BITs.Full = 0;	
		uart.send_lbuf.pRead.Val++;
		if(uart.send_lbuf.pRead.Val == uart.send_lbuf.pWrite.Val){
			uart.send_lbuf.Status.BITs.Empty = 1;
		}
		uart.send_lbuf.Wused.Val--;
	}
	return(temp);
}
static void uart_receive_dbuf_wr( unsigned short Data){
	if(uart.receive_dbuf.Status.BITs.Full == 0){
		uart.receive_dbuf.Data[uart.receive_dbuf.pWrite.Val] = Data;
		uart.receive_dbuf.Status.BITs.Empty = 0;
		uart.receive_dbuf.pWrite.Val++;
		if(uart.receive_dbuf.pRead.Val == uart.receive_dbuf.pWrite.Val)
			uart.receive_dbuf.Status.BITs.Full = 1;
		uart.receive_dbuf.Wused.Val++;
	}
}
static unsigned char uart_receive_dbuf_rd(void){
	unsigned char temp = 0;
	if(uart.receive_dbuf.Status.BITs.Empty == 0){
		temp = uart.receive_dbuf.Data[uart.receive_dbuf.pRead.Val];
		uart.receive_dbuf.Status.BITs.Full = 0;			
		uart.receive_dbuf.pRead.Val++;
		if(uart.receive_dbuf.pRead.Val == uart.receive_dbuf.pWrite.Val){
			uart.receive_dbuf.Status.BITs.Empty = 1;
		}
		uart.receive_dbuf.Wused.Val--;
	}
	return(temp);
}
static void uart_receive_lbuf_wr( unsigned short Data){
	if(uart.receive_lbuf.Status.BITs.Full == 0){
		uart.receive_lbuf.Data[uart.receive_lbuf.pWrite.Val] = Data;
		uart.receive_lbuf.Status.BITs.Empty = 0;
		uart.receive_lbuf.pWrite.Val++;
		if(uart.receive_lbuf.pRead.Val == uart.receive_lbuf.pWrite.Val)
			uart.receive_lbuf.Status.BITs.Full = 1;
		uart.receive_lbuf.Wused.Val++;
	}
}
static unsigned char uart_receive_lbuf_rd(void){
	unsigned char temp = 0;
	if(uart.receive_lbuf.Status.BITs.Empty == 0){
		temp =uart.receive_lbuf.Data[uart.receive_lbuf.pRead.Val];
		uart.receive_lbuf.Status.BITs.Full = 0;		
		uart.receive_lbuf.pRead.Val++;
		if(uart.receive_lbuf.pRead.Val == uart.receive_lbuf.pWrite.Val){
			uart.receive_lbuf.Status.BITs.Empty = 1;
		}
		uart.receive_lbuf.Wused.Val--;
	}
	return(temp);
}


//Uart Operation
static float uart_cal_chartime(unsigned int baudrate){
	float temp_chartime = 0;
	temp_chartime = uart_cal_framelength()/baudrate;
	uart.chartime = temp_chartime;
	uart.chartimex3P5 = temp_chartime*3.5;
	uart.chartimex4 = temp_chartime*4.0;
	return(temp_chartime);
};
static float uart_cal_framelength(void){
	COMLCR_T temp;
	float length = 1.0;			//1 start bit
	
	temp.Val = pADI_UART->COMLCR;
	
	if(temp.BITs.PEN)
		length += 1.0;
	switch(temp.BITs.WLS){
		case 0x00:{
			length += 5.0;
			break;
		}
		case 0x01:{
			length += 6.0;
			break;
		}
		case 0x02:{
			length += 7.0;
			break;
		}
		case 0x03:{
			length += 8.0;
			break;			
		}
		default:
			break;
	}
	if(temp.BITs.STOP){
		switch(temp.BITs.WLS){
			case 0x00:{
				length += 1.5;
				break;
			}
			case 0x01:{
				length += 2.0;
				break;
			}
			case 0x02:{
				length += 2.0;
				break;
			}
			case 0x03:{
				length += 2.0;
				break;
			}
			default:
				break;
		}
	}
	else{
		length += 1.0;
	}
	uart.framelength = (unsigned int)length;
	
	return(length);
}
static unsigned char uart_init(void){
	UrtCfg(pADI_UART, UART_BAUDRATE_INIT, COMLCR_WLS_8BITS, 0);//9600 Baud rate, 8bit Data with 1 stop bit and no parity check.
//	UrtBrk(pADI_UART, 0); // Disable Break Condition.
	UrtMod(pADI_UART,COMMCR_DTR,0);              // Setup modem bits
//Baudrate
	uart.setbaudrate(UART_BAUDRATE_INIT);
// Setup UART IRQ sources
	UrtIntCfg(pADI_UART,COMIEN_ERBFI|COMIEN_ETBEI);   // Setup UART IRQ sources	
//RS485 Dir to receive mode.
	UART_RS485_DIR = RS485_RECEIVE;		
	pADI_UART->COMCON = 0x00;		//Enable UART
	return(0);
}
static unsigned char uart_dmainit(void){
	DmaBase();
	UrtCfg(pADI_UART,B9600,COMLCR_WLS_8BITS,0);        // setup baud rate for 9600, 8-bits
	UrtMod(pADI_UART,COMMCR_DTR,0);                    // Setup modem bits
	uart.setbaudrate(UART_BAUDRATE_INIT);
	UrtIntCfg(pADI_UART,COMIEN_ERBFI|COMIEN_ETBEI|COMIEN_EDMAT|COMIEN_EDMAR);   // Setup UART IRQ sources
	//UrtDma(pADI_UART,COMIEN_EDMAT|COMIEN_EDMAR);        // Enable UART DMA interrupts	
	UrtDma(pADI_UART,COMIEN_EDMAT);        // Enable UART DMA interrupts	
	DmaPeripheralStructSetup(UARTTX_C,DMA_DSTINC_NO|DMA_SRCINC_BYTE|DMA_SIZE_BYTE);    // Enable DMA write channel	    
	DmaStructPtrInSetup(UARTRX_C,32,szPacketIn);
	DmaCycleCntCtrl(UARTRX_C,32,DMA_DSTINC_BYTE|DMA_SRCINC_NO|DMA_SIZE_BYTE|DMA_BASIC);
	DmaClr(DMARMSKCLR_UARTRX,0,0,0);
	DmaSet(0,DMAENSET_UARTRX,0,0);                      // Enable UART RX DMA primary structure	
//RS485 Dir to receive mode.
	UART_RS485_DIR = RS485_RECEIVE;		
	pADI_UART->COMCON = 0x00;		//Enable UART
	return(0);	
};
static void uart_dmasend(void){
	unsigned char status = 0;	 
	do{status = pADI_UART->COMIIR;}	while((status & 0x03)==0) ;	//No Interrupt or Transmit Complete
	//if(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]<1){		//There are more then 1 byte in the uart send buffer
	if(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]<0){		//There are more then 1 byte in the uart send buffer
		DmaStructPtrOutSetup(UARTTX_C,(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]-1),szTemp);  // Setup UARTTX source/destination pointers and number of bytes to send
		DmaCycleCntCtrl(UARTTX_C,(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]-1),DMA_DSTINC_NO| // Setup CHNL_CFG settings
			DMA_SRCINC_BYTE|DMA_SIZE_BYTE|DMA_BASIC);
		DmaClr(DMARMSKCLR_UARTTX,0,0,0);           // Disable masking of UARTTX DMA channel
		DmaSet(0,DMAENSET_UARTTX,0,                // Enable UART TX DMA channel
			DMAPRISET_UARTTX);                      // Enable UART DMA primary structure
	#ifdef __RS485_EN__
		UART_RS485_DIR = RS485_SEND;
	#endif	
		pADI_UART->COMTX = uart.send_dbuf_rd();                      // Send byte 0 to the UART to start transfer	
	}
};
static unsigned char uart_setbaudrate(unsigned int baudrate){
	
	int i1;
	int iDiv;
	uart.baudrate = baudrate;
	{				//baudrate setting
		iDiv = (pADI_CLKCTL->CLKCON1& 0x0E00);// Read UART clock as set by CLKCON1
		iDiv = iDiv>>9;
		
		switch (iDiv){
				case 0:
					iDiv = 1;
					break;
				
				case 1:
					iDiv = 2;
					break;
				
				case 2:
					iDiv = 4;
					break;
			
				case 3:
					iDiv = 8;
					break;
			
				case 4:
					iDiv = 16;
					break;
				
				case 5:
					iDiv = 32;
					break;
				
				case 6:
					iDiv = 64;
					break;	
				
				case 7:
					iDiv = 128;
					break;
				default:	 
					 break;
		}
		if ((pADI_CLKCTL->CLKSYSDIV& 0x1)==1)
			iDiv = iDiv*2;
		i1 = (16000000/(32*iDiv))/baudrate;	  // UART baud rate clock source is UARTCLK divided by 32
		pADI_UART->COMDIV = i1;
		pADI_UART->COMFBR = 0x8800|(((((2048/(32*iDiv))*16000000)/i1)/baudrate)-2048);
	}
	uart.cal_chartime(baudrate);
	uart.cal_framelength();
	return(0);
}
static void uart_send_string_int(void){ //Send a String by Uart Interrupt
	unsigned char status = 0;
	do{status = pADI_UART->COMIIR;}	while((status & 0x03)==0) ;	//No Interrupt or Transmit Complete
#ifdef __RS485_EN__
	UART_RS485_DIR = RS485_SEND;
#endif														//Set 485 to Transmit Data;
	//if(uart.send_dbuf.Status.BITs.Empty ==0){			//UartTxBuf not Empty
	if(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]){			//UartTxBuf not Empty
		pADI_UART->COMTX = uart.send_dbuf_rd();
		uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]--;
	}
}
static void uart_modbusend_string_int(void){								
	uart.status = UART_MODBUS_1stSILENCE;
	timer1.init();					// Keep silence for length of 4x Uart Character Time
	timer1.enable();
};
static void uart_normalsend_string_int(void){
	uart.status = UART_NORMAL_SEND;
	uart.send_string_int();
}
static int UrtCfgX(ADI_UART_TypeDef *pPort, int iBaud, int iBits, int iFormat){
	int i1;
	int iDiv;
			
	iDiv = (pADI_CLKCTL->CLKCON1& 0x0E00);// Read UART clock as set by CLKCON1
	iDiv = iDiv>>9;
	
	switch (iDiv)
		{
			case 0:
				iDiv = 1;
			  break;
			
			case 1:
				iDiv = 2;
			  break;
			
			case 2:
				iDiv = 4;
			  break;
		
	    case 3:
				iDiv = 8;
			  break;
		
			case 4:
				iDiv = 16;
			  break;
			
			case 5:
				iDiv = 32;
			  break;
			
			case 6:
				iDiv = 64;
			  break;	
			
			case 7:
				iDiv = 128;
			  break;
			default:	 
		     break;
			}
			if ((pADI_CLKCTL->CLKSYSDIV& 0x1)==1)
				 iDiv = iDiv*2;
	i1 = (16000000/(32*iDiv))/iBaud;	  // UART baud rate clock source is UARTCLK divided by 32
	pPort->COMDIV = i1;
	pPort->COMFBR = 0x8800|(((((2048/(32*iDiv))*16000000)/i1)/iBaud)-2048);
	//pPort->COMIEN = 0;
	pPort->COMLCR = (iFormat&0x3c)|(iBits&3);	
	return	pPort->COMLSR;
	}


static unsigned short uart_send_crc(SENDDDD_DBUF_PTR_T phead, unsigned short length){
	unsigned short CRC_REG = 0xFFFF;//Step 1
	unsigned short shift = 0;	
	SENDDDD_DBUF_PTR_T ptr;
	ptr.Val = phead.Val;
	while(length--){	
		CRC_REG ^= uart.send_dbuf.Data[ptr.Val];								//*data;		//Step 2
		for(shift = 0;shift<8;shift++){
			if(CRC_REG&0x0001){
				CRC_REG = CRC_REG >> 1;			//Step 3
				CRC_REG ^= 0xA001;
			}
			else{
				CRC_REG = CRC_REG >> 1;			//Step 3
			}
		}
		ptr.Val++;
	}
	return(CRC_REG);
}
static unsigned short uart_receive_crc(RECEIVE_DBUF_PTR_T phead, unsigned short length){
	unsigned short CRC_REG = 0xFFFF;//Step 1
	unsigned short shift = 0;	
	RECEIVE_DBUF_PTR_T ptr;
	ptr.Val = phead.Val;
	while(length--){	
		CRC_REG ^= uart.receive_dbuf.Data[ptr.Val];								//*data;		//Step 2
		for(shift = 0;shift<8;shift++){
			if(CRC_REG&0x0001){
				CRC_REG = CRC_REG >> 1;			//Step 3
				CRC_REG ^= 0xA001;
			}
			else{
				CRC_REG = CRC_REG >> 1;			//Step 3
			}
		}
		ptr.Val++;
	}
	return(CRC_REG);
}
	
	
extern void UART_Int_Handler(void){
	unsigned char temp_COMIIR = 0;
	temp_COMIIR = pADI_UART->COMIIR;
	if (temp_COMIIR & 0x4){// Receive byte
		switch(uart.status)
		{
			case UART_IDLE:	//1st byte coming.
			{
				uart.status = UART_MODBUS_RECEIVE;
				timer1.timeout(timer1.cnt_chartimex3P5);
				timer1.enable();			//Enable Timer1, start to count.
				uart.receive_dbuf_wr(pADI_UART->COMRX);
				break;
			}
			case UART_MODBUS_RECEIVE:
			{
				timer1.recount();			//Clear Timer1 and restart to count.
				uart.receive_dbuf_wr(pADI_UART->COMRX);
				break;
			}
			default:
			{
				uart.status = UART_IDLE;
				timer1.disable();
				timer1.clear_int();
				break;
			}
		}
	}
	if (temp_COMIIR & 0x2){// Transmit buffer empty One byte just transmitted.
		switch(uart.status)
		{
			case UART_NORMAL_SEND:{
				//if(uart.send_dbuf.Status.BITs.Empty ==0){			//UartTxBuf not Empty
				if(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]){		//1 FRAME was Sending
					pADI_UART->COMTX = uart.send_dbuf_rd();
					uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]--;
				}
				else{																									//1 FRAME Just Send
					uart.send_lbuf_rd();//Remove the uart send length buffer data.
		#ifdef __RS485_EN__
					for(;;){		//Wait till Transmition Complete.
						if((pADI_UART->COMLSR)&0x40)
							break;
					}
					UART_RS485_DIR = RS485_RECEIVE;
		#endif		
				}
				break;
			}
			case UART_MODBUS_DATA:{
				//if(uart.send_dbuf.Status.BITs.Empty ==0){			//UartTxBuf not Empty
				if(uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]){		//1 FRAME was Sending
					timer1.recount();
					pADI_UART->COMTX = uart.send_dbuf_rd();
					uart.send_lbuf.Data[uart.send_lbuf.pRead.Val]--;
				}
				else	//	Tx Buffer Empty, Switch 485 to Receive Mode.
				{
					uart.status = UART_MODBUS_2ndSILENCE;
					timer1.disable();
					timer1.timeout(timer1.cnt_chartimex4);
		#ifdef __RS485_EN__
					for(;;)		//Wait till Transmition Complete.
					{
						if((pADI_UART->COMLSR)&0x40)
							break;
					}
					UART_RS485_DIR = RS485_RECEIVE;
		#endif
					timer1.enable();
				}
				break;
			}
			default:
			{
				timer1.disable();
				timer1.clear_int();
				uart.status = UART_IDLE;
				break;
			}	
		}
	}
}
//Uart combined with Timer 1 to control the receiveing process.
//When Uart_Status was in Idle Mode, 1st Received data will be saved into Receive buffer, besides, the Timer 1 would be enabled.
//If next data received in 3.5 Character time, the data would be saved in the buffer, Timer 1 will be cleared and keep enabled
//Otherwise, One commend arrived for checking in the main() or Timer 1 will be cleared and disabled.
extern void DMA_UART_TX_Int_Handler(){
	volatile unsigned char ucCOMSTA0 = 0;
   volatile unsigned char ucCOMIID0 = 0;

   ucCOMIID0 = UrtIntSta(pADI_UART);                // Read UART Interrupt ID register 
	 DmaSet(DMARMSKSET_UARTTX,DMAENSET_UARTTX,0,0);   // MASK UART DMA channel to prevent further interrupts triggering - unmask when ready to re-transmit
} 

extern void DMA_UART_RX_Int_Handler(){
	volatile unsigned char ucCOMSTA0 = 0;
   volatile unsigned char ucCOMIID0 = 0;

  DmaSet(DMARMSKSET_UARTRX,DMAENSET_UARTRX,0,0);    // MASK UART DMA RX channel to prevent further interrupts triggering - unmask when ready to re-transmit 
	ucCOMIID0 = UrtIntSta(pADI_UART);                 // Read UART Interrupt ID register 
//	 ucPacketRcvd = 1;                                // Flag to indicate packet received
}
